=== modified file 'drizzled/message/command_reader.cc'
--- drizzled/message/command_reader.cc	2009-06-08 18:04:01 +0000
+++ drizzled/message/command_reader.cc	2009-06-09 06:13:27 +0000
@@ -204,17 +204,20 @@
 
   char *buffer= NULL;
   char *temp_buffer;
-  uint64_t previous_length= 0;
+  ssize_t previous_length= 0;
+  ssize_t read_bytes= 0;
+  ssize_t length= 0;
 
-  while (1)
+  /* Read in the length of the command */
+  while ((read_bytes= read(file, &length, sizeof(ssize_t))) != 0)
   {
-    uint64_t length;
-
-    /* Read the size */
-    if (read(file, &length, sizeof(uint64_t)) != sizeof(uint64_t))
-      break;
-
-    if (length > SIZE_MAX)
+    if (read_bytes == -1)
+    {
+      cerr << "Failed to read initial length." << endl;
+      exit(1);
+    }
+
+    if ((size_t) length > SIZE_MAX)
     {
       cerr << "Attempted to read record bigger than SIZE_MAX" << endl;
       exit(1);
@@ -222,26 +225,32 @@
 
     /* If we've already allocated a buffer bigger than what's needed, just zero out the buffer... */
     if (length > previous_length)
+    {
       temp_buffer= (char *) realloc(buffer, (size_t) length);
-    else
-      memset(temp_buffer, 0, sizeof(temp_buffer));
 
-    if (temp_buffer == NULL)
-    {
-      cerr << "Memory allocation failure trying to allocate " << length << " bytes."  << endl;
-      exit(1);
+      if (temp_buffer == NULL)
+      {
+        cerr << "Memory allocation failure trying to allocate " << length << " bytes."  << endl;
+        exit(1);
+      }
     }
     
     buffer= temp_buffer;
-    size_t read_bytes= 0;
 
-    /* Read the transaction */
-    if ((read_bytes= read(file, buffer, (size_t)length)) != (size_t)length)
+    /* Read the Command */
+    read_bytes= read(file, buffer, (size_t) length);
+    if ((read_bytes != (ssize_t) length))
     {
       cerr << "Could not read entire transaction. Read " << read_bytes << " bytes instead of " << length << " bytes." << endl;
       exit(1);
     }
-    command.ParseFromArray(buffer, (int) length);
+
+    if (! command.ParseFromArray(buffer, (int) length))
+    {
+      cerr << "Unable to parse command. Got error: " << command.InitializationErrorString() << endl;
+      cerr << "BUFFER: " << buffer << endl;
+      exit(1);
+    }
 
     /* Print the command */
     printCommand(command);

=== modified file 'drizzled/transaction_services.cc'
--- drizzled/transaction_services.cc	2009-06-05 19:06:04 +0000
+++ drizzled/transaction_services.cc	2009-06-09 06:49:00 +0000
@@ -138,7 +138,7 @@
     }
     ++appl_iter;
   }
-  /* If we get here, there are no active replicators or appliers */
+  /* If we get here, there are no active appliers */
   is_active= false;
 }
 
@@ -277,7 +277,10 @@
   push(&command);
 }
 
-void TransactionServices::updateRecord(Session *in_session, Table *in_table, const unsigned char *, const unsigned char *)
+void TransactionServices::updateRecord(Session *in_session,
+                                       Table *in_table, 
+                                       const unsigned char *old_record, 
+                                       const unsigned char *new_record)
 {
   using namespace drizzled::message;
   
@@ -300,7 +303,56 @@
    * Now we construct the specialized UpdateRecord command inside
    * the Command container...
    */
-  //UpdateRecord *change_record= command.mutable_update_record();
+  UpdateRecord *change_record= command.mutable_update_record();
+
+  Field *current_field;
+  Field **table_fields= in_table->field;
+  String *string_value= new (in_session->mem_root) String(100); /* 100 initially. field->val_str() is responsible for re-adjusting */
+  string_value->set_charset(system_charset_info);
+
+  Table::Field *cur_field;
+
+  while ((current_field= *table_fields++) != NULL) 
+  {
+    /*
+     * The below really should be moved into the Field API and Record API.  But for now
+     * we do this crazy pointer fiddling to figure out if the current field
+     * has been updated in the supplied record raw byte pointers.
+     */
+    const unsigned char *old_ptr= (const unsigned char *) old_record + (ptrdiff_t) (current_field->ptr - in_table->record[0]); 
+    const unsigned char *new_ptr= (const unsigned char *) new_record + (ptrdiff_t) (current_field->ptr - in_table->record[0]); 
+
+    uint32_t field_length= current_field->pack_length(); /** @TODO This isn't always correct...check varchar diffs. */
+
+    if (memcmp(old_ptr, new_ptr, field_length) != 0)
+    {
+      /* Field is changed from old to new */
+      cur_field= change_record->add_update_field();
+      cur_field->set_name(std::string(current_field->field_name));
+      cur_field->set_type(Table::Field::VARCHAR); /* @TODO real types! */
+      string_value= current_field->val_str(string_value);
+      change_record->add_after_value(std::string(string_value->c_ptr()));
+      string_value->free(); /* I wish there was a clear() method... */
+    }
+
+    /* 
+     * Add the WHERE clause values now...the fields which return true
+     * for isReadSet() are in the WHERE clause.  For tables with no
+     * primary or unique key, all fields will be returned.
+     */
+    if (current_field->isReadSet())
+    {
+      cur_field= change_record->add_where_field();
+      cur_field->set_name(std::string(current_field->field_name));
+      cur_field->set_type(Table::Field::VARCHAR); /* @TODO real types! */
+      string_value= current_field->val_str(string_value);
+      change_record->add_where_value(std::string(string_value->c_ptr()));
+      string_value->free(); /* I wish there was a clear() method... */
+    }
+  }
+
+  if (string_value)
+    delete string_value; /* Is this needed with memroot allocation? */
 
   push(&command);
 }

=== modified file 'drizzled/transaction_services.h'
--- drizzled/transaction_services.h	2009-06-05 19:06:04 +0000
+++ drizzled/transaction_services.h	2009-06-09 06:35:35 +0000
@@ -3,6 +3,10 @@
  *
  *  Copyright (C) 2008-2009 Sun Microsystems
  *
+ *  Authors:
+ *
+ *    Jay Pipes <joinfu@sun.com>
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; version 2 of the License.
@@ -17,8 +21,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef DRIZZLED_REPLICATOR_H
-#define DRIZZLED_REPLICATOR_H
+#ifndef DRIZZLED_TRANSACTION_SERVICES_H
+#define DRIZZLED_TRANSACTION_SERVICES_H
 
 #include "drizzled/atomics.h"
 #include <vector>
@@ -160,8 +164,13 @@
    *
    * @param Pointer to the Session which has updated a record
    * @param Pointer to the Table containing update information
+   * @param Pointer to the raw bytes representing the old record/row
+   * @param Pointer to the raw bytes representing the new record/row 
    */
-  void updateRecord(Session *in_session, Table *in_table, const unsigned char *, const unsigned char *);
+  void updateRecord(Session *in_session, 
+                    Table *in_table, 
+                    const unsigned char *old_record, 
+                    const unsigned char *new_record);
   /**
    * Creates a new DeleteRecord GPB message and pushes it to
    * replicators.
@@ -188,20 +197,4 @@
 
 } /* end namespace drizzled */
 
-#ifdef oldcode
-/* todo, fill in this API */
-/* these are the functions called by the rest of the drizzle server
-   to do whatever this plugin does. */
-bool replicator_session_init (Session *session);
-bool replicator_write_row(Session *session, Table *table);
-bool replicator_update_row(Session *session, Table *table,
-                           const unsigned char *before,
-                           const unsigned char *after);
-bool replicator_delete_row(Session *session, Table *table);
-
-/* The below control transactions */
-bool replicator_end_transaction(Session *session, bool autocommit, bool commit);
-bool replicator_prepare(Session *session);
-bool replicator_statement(Session *session, const char *query, size_t query_length);
-#endif /* oldcode */
-#endif /* DRIZZLED_REPLICATOR_H */
+#endif /* DRIZZLED_TRANSACTION_SERVICES_H */

=== modified file 'plugin/serial_event_log/serial_event_log.cc'
--- plugin/serial_event_log/serial_event_log.cc	2009-06-05 19:51:55 +0000
+++ plugin/serial_event_log/serial_event_log.cc	2009-06-09 03:49:16 +0000
@@ -28,9 +28,22 @@
  *
  * @details
  *
- * Currently, this is a very simple implementation.  We have a global
- * lock on the file writer and write the events as the are received, 
- * first writing a 64-bit length and then the serialized transaction/command.
+ * Currently, the log file uses this implementation:
+ *
+ * We have an atomic off_t called log_offset which keeps track of the 
+ * offset into the log file for writing the next Command.
+ *
+ * When writing a Command to the log, we calculate the length of the 
+ * Command to be written.  We then increment log_offset by the length
+ * of the Command plus sizeof(uint64_t) and store this new offset in a 
+ * local off_t called cur_offset (see SerialEventLog::apply().  This 
+ * compare and set is done in an atomic instruction.
+ *
+ * We then adjust the local off_t (cur_offset) back to the original
+ * offset by subtracting the length and sizeof(uint64_t).
+ *
+ * We then first write a 64-bit length and then the serialized transaction/command
+ * to our log file at our local cur_offset.
  *
  * @todo
  *
@@ -104,17 +117,17 @@
 void SerialEventLog::apply(drizzled::message::Command *to_apply)
 {
   std::string buffer; /* Buffer we will write serialized command to */
-  size_t length;
-  size_t written;
+  ssize_t length;
+  ssize_t written;
   off_t cur_offset;
 
   to_apply->SerializeToString(&buffer);
-  length= buffer.length();
+  length= (ssize_t) buffer.length();
 
   /*
    * Do an atomic increment on the offset of the log file position
    */
-  cur_offset= log_offset+= (sizeof(uint64_t) + length);
+  cur_offset= log_offset+= (off_t) (sizeof(ssize_t) + length);
   /** 
    * @TODO
    *
@@ -136,7 +149,7 @@
    * We adjust cur_offset back to the original log_offset before
    * the increment above...
    */
-  cur_offset-= (sizeof(uint64_t) + length);
+  cur_offset-= (off_t) (sizeof(ssize_t) + length);
 
   /* 
    * Quick safety...if an error occurs below, the log file will
@@ -147,13 +160,13 @@
     return;
 
   written= pwrite(log_file, &length, sizeof(uint64_t), cur_offset);
-  cur_offset+= written;
-  if (unlikely(written != sizeof(uint64_t)))
+  cur_offset+= (off_t) written;
+  if (unlikely(written != sizeof(ssize_t)))
   {
     errmsg_printf(ERRMSG_LVL_ERROR, 
                   _("Failed to write full size of command.  Tried to write %" PRId64 ", but only wrote %" PRId64 ".  Error: %s"), 
-                  (uint64_t) length, 
-                  (uint64_t) written, 
+                  (int64_t) length, 
+                  (int64_t) written, 
                   strerror(errno));
     state= CRASHED;
     is_active= false;
@@ -172,12 +185,11 @@
   {
     errmsg_printf(ERRMSG_LVL_ERROR, 
                   _("Failed to write full serialized command.  Tried to write %" PRId64 ", but only wrote %" PRId64 ".  Error: %s"), 
-                  (uint64_t) length, 
-                  (uint64_t) written, 
+                  (int64_t) length, 
+                  (int64_t) written, 
                   strerror(errno));
     state= CRASHED;
     is_active= false;
-    return;
   }
 }
 

=== added file 'tests/suite/serial_event_log/r/multi_insert.result'
--- tests/suite/serial_event_log/r/multi_insert.result	1970-01-01 00:00:00 +0000
+++ tests/suite/serial_event_log/r/multi_insert.result	2009-06-09 06:35:35 +0000
@@ -0,0 +1,14 @@
+DROP TABLE IF EXISTS t1;
+CREATE TABLE t1 (
+id INT NOT NULL
+, padding VARCHAR(200) NOT NULL
+);
+INSERT INTO t1 VALUES (1, "I love testing."), (2, "I hate testing.");
+/*  */
+/* SID: 1 XID: 0 */ DROP TABLE IF EXISTS t1;
+/*  */
+/* SID: 1 XID: 0 */ CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL );
+/*  */
+/* SID: 1 XID: 3 */ INSERT INTO `test`.`t1` (`id`, `padding`) VALUES ("1", "I love testing.");
+/*  */
+/* SID: 1 XID: 3 */ INSERT INTO `test`.`t1` (`id`, `padding`) VALUES ("2", "I hate testing.");

=== added file 'tests/suite/serial_event_log/r/update.result'
--- tests/suite/serial_event_log/r/update.result	1970-01-01 00:00:00 +0000
+++ tests/suite/serial_event_log/r/update.result	2009-06-09 06:49:00 +0000
@@ -0,0 +1,22 @@
+DROP TABLE IF EXISTS t1;
+CREATE TABLE t1 (
+id INT NOT NULL
+, padding VARCHAR(200) NOT NULL
+, PRIMARY KEY (id)
+);
+INSERT INTO t1 VALUES (1, "I love testing.");
+INSERT INTO t1 VALUES (2, "I hate testing.");
+UPDATE t1 SET padding= "I REALLY love testing." WHERE id = 1;
+UPDATE t1 SET padding= "I REALLY hate testing." WHERE id = 2;
+/*  */
+/* SID: 1 XID: 0 */ DROP TABLE IF EXISTS t1;
+/*  */
+/* SID: 1 XID: 0 */ CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL , PRIMARY KEY (id) );
+/*  */
+/* SID: 1 XID: 3 */ INSERT INTO `test`.`t1` (`id`, `padding`) VALUES ("1", "I love testing.");
+/*  */
+/* SID: 1 XID: 4 */ INSERT INTO `test`.`t1` (`id`, `padding`) VALUES ("2", "I hate testing.");
+/*  */
+/* SID: 1 XID: 5 */ UPDATE `test`.`t1` SET `padding` = "I REALLY love testing." WHERE `id` = "1"
+/*  */
+/* SID: 1 XID: 6 */ UPDATE `test`.`t1` SET `padding` = "I REALLY hate testing." WHERE `id` = "2"

=== removed file 'tests/suite/serial_event_log/r/update.result'
--- tests/suite/serial_event_log/r/update.result	2009-06-09 02:18:29 +0000
+++ tests/suite/serial_event_log/r/update.result	1970-01-01 00:00:00 +0000
@@ -1,17 +0,0 @@
-DROP TABLE IF EXISTS t1;
-CREATE TABLE t1 (id INT NOT NULL, data INT);
-INSERT INTO t1 VALUES (1, 1), (2, 2), (3, 3);
-UPDATE t1 SET data = 9 WHERE id = 2;
-/*  */
-/* SID: 1 XID: 0 */ DROP TABLE IF EXISTS t1;
-/*  */
-/* SID: 1 XID: 0 */ CREATE TABLE t1 (id INT NOT NULL, data INT);
-/*  */
-/* SID: 1 XID: 3 */ INSERT INTO `test`.`t1` (`id`, `data`) VALUES ("1", "1");
-/*  */
-/* SID: 1 XID: 3 */ INSERT INTO `test`.`t1` (`id`, `data`) VALUES ("2", "2");
-/*  */
-/* SID: 1 XID: 3 */ INSERT INTO `test`.`t1` (`id`, `data`) VALUES ("3", "3");
-/*  */
-/* SID: 1 XID: 4 */ UPDATE `test`.`t1` SET  WHERE id = 2; 
-

=== renamed file 'tests/suite/serial_event_log/t/simple-master.opt' => 'tests/suite/serial_event_log/t/insert-master.test'
=== renamed file 'tests/suite/serial_event_log/t/simple.test' => 'tests/suite/serial_event_log/t/insert.test'
--- tests/suite/serial_event_log/t/simple.test	2009-06-03 20:31:33 +0000
+++ tests/suite/serial_event_log/t/insert.test	2009-06-09 06:35:35 +0000
@@ -1,5 +1,5 @@
 # 
-# Simple test of the serial event log
+# Simple test of the serial event log for single INSERT statements
 # 
 # We create a table and insert some records
 # into it.

=== renamed file 'tests/suite/serial_event_log/t/insert.test' => 'tests/suite/serial_event_log/t/insert.test.moved'
=== added file 'tests/suite/serial_event_log/t/multi_insert-master.opt'
--- tests/suite/serial_event_log/t/multi_insert-master.opt	1970-01-01 00:00:00 +0000
+++ tests/suite/serial_event_log/t/multi_insert-master.opt	2009-06-09 06:35:35 +0000
@@ -0,0 +1,1 @@
+--default-replicator-enable --serial-event-log-enable --scheduler=multi_thread

=== added file 'tests/suite/serial_event_log/t/multi_insert.test'
--- tests/suite/serial_event_log/t/multi_insert.test	1970-01-01 00:00:00 +0000
+++ tests/suite/serial_event_log/t/multi_insert.test	2009-06-09 06:35:35 +0000
@@ -0,0 +1,23 @@
+# 
+# Simple test of the serial event log for multi-value INSERT statements
+# 
+# We create a table and insert some records
+# into it.
+# 
+# We then use the transaction_reader in drizzled/message/ to read the events.
+#
+
+--disable_warnings
+DROP TABLE IF EXISTS t1;
+--enable_warnings
+
+CREATE TABLE t1 (
+  id INT NOT NULL
+, padding VARCHAR(200) NOT NULL
+);
+
+INSERT INTO t1 VALUES (1, "I love testing."), (2, "I hate testing.");
+
+# Read in the event.log.  We must cut out all the Timestamp: XXXXXX
+# because they are not deterministic.
+--exec ../drizzled/message/command_reader var/master-data/event.log | sed 's/Timestamp: [0-9]*//g'

=== added file 'tests/suite/serial_event_log/t/update-master.opt'
--- tests/suite/serial_event_log/t/update-master.opt	1970-01-01 00:00:00 +0000
+++ tests/suite/serial_event_log/t/update-master.opt	2009-06-09 06:35:35 +0000
@@ -0,0 +1,1 @@
+--default-replicator-enable --serial-event-log-enable --scheduler=multi_thread

=== renamed file 'tests/suite/serial_event_log/t/update-master.opt' => 'tests/suite/serial_event_log/t/update-master.opt.moved'
=== added file 'tests/suite/serial_event_log/t/update.test'
--- tests/suite/serial_event_log/t/update.test	1970-01-01 00:00:00 +0000
+++ tests/suite/serial_event_log/t/update.test	2009-06-09 06:49:00 +0000
@@ -0,0 +1,28 @@
+# 
+# Simple test of the serial event log
+# 
+# We create a table and insert some records
+# into it.
+# 
+# We then use the transaction_reader in drizzled/message/ to read the events.
+#
+
+--disable_warnings
+DROP TABLE IF EXISTS t1;
+--enable_warnings
+
+CREATE TABLE t1 (
+  id INT NOT NULL
+, padding VARCHAR(200) NOT NULL
+, PRIMARY KEY (id)
+);
+
+INSERT INTO t1 VALUES (1, "I love testing.");
+INSERT INTO t1 VALUES (2, "I hate testing.");
+
+UPDATE t1 SET padding= "I REALLY love testing." WHERE id = 1;
+UPDATE t1 SET padding= "I REALLY hate testing." WHERE id = 2;
+
+# Read in the event.log.  We must cut out all the Timestamp: XXXXXX
+# because they are not deterministic.
+--exec ../drizzled/message/command_reader var/master-data/event.log | sed 's/Timestamp: [0-9]*//g'

=== renamed file 'tests/suite/serial_event_log/t/update.test' => 'tests/suite/serial_event_log/t/update.test.moved'
